#!/usr/bin/perl -w
#
#  avast! VPS update script
#  Put a call to this in your root crontab
#
#  Copyright(c) 2004-2005   ALWIL Software
#
# This script will check if the VPS file defined in your configuration 
# is up to date. If not, it will download the latest VPS from the avast
# website. It will call 'avastvpsreload' to install the downloaded VPS
# in its right place.
# --------------------------------------------------------------------
# Adapted by Mike Vervuren (avastvpsupdate@vervuren.com)
#
# By using this code you agree to indemnify Mike Vervuren 
# from any liability that might arise from its use. :-)
# - I did my best but I'm only human.
#
# Version 0.2 (Feb 08, 2005) 
# - Improved error reporting/logging.
#   - added package logger in order to write to the logfile
# - Changed config file handling
# - Use LWP::UserAgent to get the files from ALWIL Software
#
# Version 1.0 (Sun May 22 22:42:04 CEST 2005)
# Adapted to libavastengine-4.6.0b and avast v1.0.8_2
# This version is not backwards compatible. This is version 1.
# Changes:
# - changed Avast4TmpDir to Avast4TempDir. (see 'man avastengine.conf')
# - check environment for avast settings (see 'man avastengine.conf')
# - check for $ENV{HOME}/.avastengine.conf
# - Re-introduced use of wget if LWP::UserAgent is not installed
# - Use 'avastvpsreload' to put the VPS into place. This will
#   make sure al running avast programs will get notified.
# - Create an empty VPS file if none exists.
# - Added scriptname and pid to loglines in logger package
# - Check for a writable tmp directory
#
# Version 1.1 (Tue May 31 11:59:21 CEST 2005)
# Changes:
# - logging directory changed to /var/log/avast4
# - 'avastvpsreload' is now used to check current VPS file version
# - enabled installation of the VPS file to a global avast! kernel
#   directory when 'avastvpsreload' is started by root user
#
# Version 1.2 (Tue Jul 19 23:29:17 CEST 2005)
# Changes:
# - check for a connection to the VPS file server
# - fixed bugs
#
# Version 1.3 (Fri Nov 4 11:01:43 CET 2005)
# Changes:
# - reading configuration from /etc/avastvpsupdate.conf file or
#   /usr/local/etc/avastvpsupdate.conf or ~/.avastvpsupdate.conf file
# - support for HTTP proxy connection
# - added locking mechanism so one user can run the program only once
# - other improvements
#
# Version 1.4 (Tue Nov 22 20:34:02 CET 2005)
# Changes:
# - Skipped using of URI::Split Perl module
# - fixed parsing wget output
#

use strict;
my $VERSION=1.4;
# --------------------------------------------------------------------
# CONFIG
# --------------------------------------------------------------------
# location of binaries like avastvpsreload and maybe wget.
$ENV{PATH} = "/bin:/usr/bin:/usr/local/bin";

# location of the files
my $avastconfigfile		= "/etc/avastengine.conf";
my $avastupdateconfigfile	= "/etc/avastvpsupdate.conf";


# The update config file will be checked for the following variables.
my $updateconfig	= {
  -update => "1",
  -vpslocation  => "http://files.avast.com/files/latest/",
  -netacc => "0",
  -netip => "192.168.1.1",
  -netport => "8080",
  -netuser => "",
  -netpwd => "",
};

# The config file will be checked for the following variables. We
# set the default values here. The environment will be checked for
# these variables too (see 'man avastengine.conf').
my $config		= {
  -avast4logdir  => "/var/log/avast4",
  -avast4datadir => "/var/lib/avast4",
  -avast4tempdir => "/var/tmp",
};


# --------------------------------------------------------------------
# debugging mode - for development only
my $debug = 0;
# --------------------------------------------------------------------
# END CONFIG  
# There should be no changes needed below this point.
######################################################################

# --------------------------------------------------------------------
# CPAN modules (use 'perl -MCPAN -e "shell"' to install them)
# --------------------------------------------------------------------
use Net::Ping;
use Digest::MD5;
use File::Spec::Functions qw( catfile );
use File::Copy;
use FileHandle;
use FindBin qw($Script);
use URI;

# --------------------------------------------------------------------
# verbose mode on if we're run from an interactive console. 
my $verbose = 0;
if (-t STDIN) {
  $verbose++;
  $|=1;
}


# --------------------------------------------------------------------
# Read config from avastupdateconfigfile
# --------------------------------------------------------------------
# check which updateconfigfile to use
if ( -f $ENV{'HOME'} . "/.avastvpsupdate.conf") {
  print "Updating userconfig detected.\n" if $debug;
  $avastupdateconfigfile = $ENV{'HOME'} . "/.avastvpsupdate.conf";  
}
if ( ! -f "/etc/avastvpsupdate.conf" && -f "/usr/local/etc/avastvpsupdate.conf") {
  print "/usr/local/etc/avastvpsupdate.conf updating config detected.\n" if $debug;
  $avastupdateconfigfile = "/usr/local/etc/avastvpsupdate.conf";
}

if ( -f $avastupdateconfigfile ) {
  open UPDATECONF, "<" . $avastupdateconfigfile 
    or die "Can't read update configfile '" . $avastupdateconfigfile . "':$!\n";
  my @updatelines = <UPDATECONF>;
  close UPDATECONF;

  foreach (@updatelines) {
    chomp;
    # skip empty lines and lines with only comment
    next if (/^\s*#.*/ or /^\s*$/);
    # strip comment from lines (if there's comment at the end of a line)
    s/\s+#.*$//;
    # remove all whitespaces from the line
    s/\s+//g;
    # get the name/value pairs from the line
    if (/^(.*)=(.*)$/) {
      my ($name,$value) = ($1,$2);
      # if there's a value for a configname we use, we override 
      # the default value with the value from the config
      if (exists $updateconfig->{"-" . lc $name}) {
        $updateconfig->{"-" . lc $name} = $value;
        print "Set '$name' to '$value'\n" if $debug;
      }
    }
  }
}

# --------------------------------------------------------------------
# Read config from avastconfigfile
# --------------------------------------------------------------------
# check which configfile to use
if ( -f $ENV{'HOME'} . "/.avastengine.conf") {
  print "Userconfig detected.\n" if $debug;
  $avastconfigfile = $ENV{'HOME'} . "/.avastengine.conf";
}
if ( ! -f "/etc/avastengine.conf" && -f "/usr/local/etc/avastengine.conf") {
  print "/usr/local/etc/avastengine.conf engine config detected.\n" if $debug;
  $avastconfigfile = "/usr/local/etc/avastengine.conf";
}

if ( -f $avastconfigfile ) {
  open CONF, "<" . $avastconfigfile 
    or die "Can't read configfile '" . $avastconfigfile . "':$!\n";
  my @lines = <CONF>;
  close CONF;

  foreach (@lines) {
    chomp;
    # skip empty lines and lines with only comment
    next if (/^\s*#.*/ or /^\s*$/);
    # strip comment from lines (if there's comment at the end of a line)
    s/\s+#.*$//;
    # remove all whitespaces from the line
    s/\s+//g;
    # get the name/value pairs from the line
    if (/^(.*)=(.*)$/) {
      my ($name,$value) = ($1,$2);
      # if there's a value for a configname we use, we override 
      # the default value with the value from the config
      if (exists $config->{"-" . lc $name}) {
        if (defined $ENV{$name}) {
  	print "Override config option with environment variable\n" 
	  if $debug;
          $value = $ENV{$name};
        }
        $config->{"-" . lc $name} = $value;
        print "Set '$name' to '$value'\n" if $debug;
      }
    }
  }
}


# --------------------------------------------------------------------
# check if the given Avast4TempDir exists and is writable
# --------------------------------------------------------------------
if (-e $config->{-avast4tempdir}) {
  if ( -f $config->{-avast4tempdir} ) {
	die "The Avast4TempDir '" . $config->{-avast4tempdir}. "' is a file\n";
  }
  unless (-w $config->{-avast4tempdir}) {
	die "The Avast4TempDir '" . $config->{-avast4tempdir}. "' is not writable\n";
  }
} 
else {
  # if you like you can create your tmp dir here.
  # the code would be something like this
  #mkdir $config->{-avast4tempdir},0022 
  #  or die "Could not create Avast4TempDir '" . $config->{-avast4tempdir} 
  #         . "':$!\n"; 
  die "The Avast4TempDir '" . $config->{-avast4tempdir}. "' does not exist\n";
}

# --------------------------------------------------------------------
# Start writing to the logfile (this uses the logger module included
# in this file).
# Warn if we can't write to the logfile, but we still want to update
# the vps, so keep running.
# --------------------------------------------------------------------
my $logfilename = catfile ($config->{-avast4logdir}, $Script . ".log");
print "Logging to '$logfilename'\n" if $debug;

my $updatelog = new logger($logfilename);

# this will move the logfile to logfile.old if we exceed the given size
# of the logfile.
$updatelog->maxsize() or die $logger::errstr . "\n";

$updatelog->write("----- $Script started -----") 
  or warn $logger::errstr; 

# --------------------------------------------------------------------
# check if VPS updating is enabled
# --------------------------------------------------------------------
if ( $updateconfig->{-update} == "0" ) {
    my $message = "VPS updating is disabled";
    print $message .  "\n" if $verbose;
    $updatelog->write($message) or warn $logger::errstr;
    exit (1);
}

# --------------------------------------------------------------------
# check if the given location is accessible
# --------------------------------------------------------------------
my $uri = new URI "$updateconfig->{-vpslocation}";
my $host_;
if ( $uri->scheme ) {
	my $scheme_;
	my $rest_;
	($scheme_,$host_,$rest_) = split('//?',$uri,3);
} else {
	my @myuri = split('/',$uri);
	$host_ = $myuri[0];
}

my $hostaccessible = 0;
my $p;
$p = Net::Ping->new("icmp");
if ( $p->ping($host_) ) { $hostaccessible = 1; }
$p->close();
if ( $hostaccessible == 0 ) {
    my $message = "Can't access host: $host_";
    print $message .  "\n" if $verbose;
    $updatelog->write($message) or warn $logger::errstr;
    exit (1);
}
print "Host $host_ accessible\n" if $debug;

#my $vps_web_location    	= "http://files.avast.com/files/latest/400.vps";
#my $md5_web_location    	= "http://files.avast.com/files/latest/400vps.md5";
my $vps_web_location = URI->new_abs( "400.vps", $uri );
my $md5_web_location = URI->new_abs( "400vps.md5", $uri );

# --------------------------------------------------------------------
# check if the lock file exists
# --------------------------------------------------------------------
my $vps_lock_fname = catfile( $config->{-avast4tempdir}, 'avastvpsupdate.lock' );
print "New VPS file lock will be '$vps_lock_fname'\n" if $debug;

if ( -e $vps_lock_fname ) {
    my $oldpid;
    $oldpid=`cat $vps_lock_fname`;
    if ( `ps -p $oldpid | grep $oldpid` ) {
        my $message = "VPS updating script already running at pid $oldpid";
        print $message .  "\n" if $verbose;
        $updatelog->write($message) or warn $logger::errstr;
        exit (1);
    }
    # process is dead; remove lock file
    unlink $vps_lock_fname;
}

# --------------------------------------------------------------------
# I had some trouble installing LWP::UserAgent (got it working though
# :-) ). I recon other users may have the same problems so we try using
# LWP::UserAgent but if it fails we will use wget. 
# Tip for Trustix users: Check out the precompiled perl packages
# using 'swup --search-package perl').
# --------------------------------------------------------------------
my $uselwp = 0;
eval "use LWP::UserAgent";
unless ($@) {
  $uselwp++;
  my $lwpvers = "";
  if ( defined $LWP::UserAgent::VERSION ) { $lwpvers = $LWP::UserAgent::VERSION ; }
  print "Using LWP::UserAgent version: $lwpvers\n" if $debug;
}
unless ($uselwp) {
  # ------------------------------------------------------------------
  # get wget version. Die if we can't run wget.
  # ------------------------------------------------------------------
  my $wget = ' ';
  $wget = `wget --version 2>&1`;
  if ($?) {
    die "Can't run wget:'$wget'\n.". 
        "Install the perl LWP::UserAgent module " . 
	"or install 'wget' in '" . $ENV{PATH} . "'\n"; 
  }
  my $wget_version = (split "\n",$wget)[0];
  print "Using wget '$wget_version' to get new VPS update\n" if $verbose;
}

# --------------------------------------------------------------------
# function to get file from webpage
# will return 0 on failure,1 on success
# --------------------------------------------------------------------
sub GetInetFile($$;) {
  my $url = shift;
  my $dest = shift;

  if ($uselwp) {
    my $ua = LWP::UserAgent->new;
    my $req = HTTP::Request->new(GET => $url);
    if ( $updateconfig->{-netacc} == "2" ) {
       my $neturi;
       $neturi = uri_join( "http", "$updateconfig->{-netip}:$updateconfig->{-netport}" );
       $ua->proxy( http => "$neturi" );

       if( ! -z "$updateconfig->{-netuser}" ) {
          $req->proxy_authorization( $updateconfig->{-netuser}, $updateconfig->{-netpwd} );
       }
    }
    print "Downloading $url => $dest.." if $verbose;
    $ua->agent("$Script/$VERSION " . $ua->agent);
    my $result = $ua->request($req, $dest);

    if ($result->is_success) {
      print "LWP result " . $result->content_length 
	 . " bytes [" . $result->code  . " OK]\n" if $verbose;
    } 
    else {
       my $message = "Error downloading '$url':" .
                     $result->status_line;
       print $message .  "\n" if $verbose;
       $updatelog->write($message) or warn $logger::errstr;
       return 0;
    }
  } 
  else {
    my $safeurl = quotemeta($url);
    my $command = "wget --server-response --output-document=${dest}";
    # check for http proxy
    if ( $updateconfig->{-netacc} == "2" ) {
      my $neturi = uri_join( "http", "$updateconfig->{-netip}:$updateconfig->{-netport}" );
      $ENV{http_proxy}="$neturi";
      $command = "wget --server-response --output-document=${dest} --proxy=on";
      if( ! -z "$updateconfig->{-netuser}" ) {
        $command = "$command --proxy-user=\"$updateconfig->{-netuser}\" --proxy-passwd=\"$updateconfig->{-netpwd}\"";
      }
    }
    $command = "$command ${url}";

    my $result = `$command 2>&1`; 
    # ----------------------------------------------------------------
    # We can't trust the exit code for wget :-( so we run wget with
    # server-response enabled and check for the '200 OK' code.
    # ----------------------------------------------------------------
    if ($result =~ m/HTTP/) {
      my @result = split "\n",$result;
      my %result;
      foreach (@result) {
        next unless  /^\s*\d*\s+(\S+)\s+(.*)$/;
        last if /^\s*$/;
        my ($mes,$val) = ($1,$2);
        $mes =~ s/://;
        $result{$mes} = $val;
      }

      my $http_res;
      if ( defined $result{'HTTP/1.1'} ) { $http_res = $result{'HTTP/1.1'}; }
      else { $http_res = $result{'HTTP/1.0'}; }

      if ($http_res =~ /200 OK/) {
        print "wget result: " . $result{'Content-Length'} 
		 . " bytes [" . $http_res ."] \n" if $verbose;
      }
      else {
        my $message = "Error downloading '$url': " .  $http_res ; 
        print $message .  "\n" if $verbose;
        $updatelog->write($message) or warn $logger::errstr;
        return 0;
      }
    } 
    else {
      my $message = "Error running wget '$command':$result";
      print $message .  "\n" if $verbose;
      $updatelog->write($message) or warn $logger::errstr;
      return 0;
    }
  }
  return 1;
}

# --------------------------------------------------------------------
# set some variables
# --------------------------------------------------------------------
# the location of the md5 checksum we're going to download
my $vps_md5_tmp_fname = catfile( $config->{-avast4tempdir}, '400vps.md5' );
print "MD5 checksum will be stored in '$vps_md5_tmp_fname'\n" if $debug;

# the location of the live virusdatabase files
my $vps_fname = catfile( $config->{-avast4datadir}, '400.vps' );
print "VPS file is located in '$vps_fname'\n" if $debug;

# the location of the virusdatabase file we're going to download
my $vps_tmp_fname = catfile( $config->{-avast4tempdir}, '400.vps' );
print "New VPS file will be stored in '$vps_tmp_fname'\n" if $debug;


# --------------------------------------------------------------------
# Add process PID to lock file
# --------------------------------------------------------------------
open LOCKFILE, ">${vps_lock_fname}";
print LOCKFILE "$$";
close LOCKFILE;

# --------------------------------------------------------------------
# Remove the old md5 file. First check if it exists so we can 
# generate an error and die if we can't remove it.
# --------------------------------------------------------------------
if ( -f $vps_md5_tmp_fname) {
  print "Removing old MD5 file $vps_md5_tmp_fname\n" if $debug;
  unless ( unlink $vps_md5_tmp_fname ) {
    my $message = "Could not remove the old md5 file '" 
                . $vps_md5_tmp_fname . "':$!";
    $updatelog->write($message) or warn $logger::errstr; 

    unlink ${vps_lock_fname};
    die $message . "\n";
  }
}

# --------------------------------------------------------------------
# Get the file with the new md5 checksum.
# No use continuing when we can't get the checksum.
# --------------------------------------------------------------------
print "Getting md5 checksum for new VPS\n" if $verbose;
unless (GetInetFile($md5_web_location, $vps_md5_tmp_fname)) {
  $updatelog->write("Error: cannot download VPS MD5 file.")
    or warn $logger::errstr;

  unlink ${vps_lock_fname};
  die "Error: cannot download the latest VPS MD5 file\n";
}

# --------------------------------------------------------------------
# Read the checksum from the file
# checksum are the first 32 bites in the file
# --------------------------------------------------------------------
my $newvpsmd5;
if (open  FILE, "<" . $vps_md5_tmp_fname ) {
  my $bytes = read(FILE, $newvpsmd5, 32 );
  close(FILE);
  unless (defined $bytes and $bytes == 32) {
    my $message = "Error: Couldn't read first 32 bytes from '" 
                . $vps_md5_tmp_fname . "':$!";
    $updatelog->write($message) or warn $logger::errstr;

    unlink ${vps_lock_fname};
    die $message . "\n"; 
  }
  print "Read $bytes bytes from file\n" if $debug;
  print "Removing MD5 file $vps_md5_tmp_fname\n" if $debug;
  unless ( unlink $vps_md5_tmp_fname ) {
    my $message = "WARNING:Couldn't remove md5 file '" 
                . $vps_md5_tmp_fname . "':$!";
    $updatelog->write($message) or warn $logger::errstr; 
    warn $message . "\n";
  }
} 
else {
    my $message = "Error: cannot read VPS MD5 file '"
                  .$vps_md5_tmp_fname . "': $!";
    $updatelog->write($message) or warn $logger::errstr;

    unlink ${vps_lock_fname};
    die $message . "\n";
}

# --------------------------------------------------------------------
# Get checksum of local VPS 
# --------------------------------------------------------------------
my $oldvpsmd5;
if ( -f $vps_fname) {
  unless (open FILE, "<" . $vps_fname) {
    my $message = "Error: Couldn't read '" 
                . $vps_fname . "':$!";
    $updatelog->write($message) or warn $logger::errstr;

    unlink ${vps_lock_fname};
    die $message . "\n"; 
  }
  binmode(FILE);
  $oldvpsmd5 = Digest::MD5->new->addfile(*FILE)->hexdigest();
  close FILE;
} 

# --------------------------------------------------------------------
# check if we need to download.
# --------------------------------------------------------------------
if (defined $oldvpsmd5 and ( $newvpsmd5 eq $oldvpsmd5 ) ) {
  print "No VPS download required\n" if $verbose;
  $updatelog->write("No VPS download required.");
}
else { 
  $updatelog->write("VPS file changed, downloading latest.");
  # Get new VPS file
  print "Getting new VPS\n" if $verbose;
  if (GetInetFile($vps_web_location, $vps_tmp_fname)) {
    $updatelog->write("VPS file downloaded.");
    # make the vps file worldreadable so al users can enjoy avast
    chmod 0644,$vps_tmp_fname;
    # ----------------------------------------------------------------
    # Check checksum from downloaded file
    # ----------------------------------------------------------------
    unless (open VPS, "<" . $vps_tmp_fname) {
      my $message = "Error: Couldn't read '" 
	          . $vps_tmp_fname . "':$!";
      $updatelog->write($message) or warn $logger::errstr;

      unlink ${vps_lock_fname};
      die $message . "\n"; 
    }
    binmode(VPS);
    my $downvpsmd5 = Digest::MD5->new->addfile(*VPS)->hexdigest();
    close VPS;

    unless ($newvpsmd5 eq $downvpsmd5) {
      my $message = "Error: The MD5 checksum for the downloaded " 
                  . " file isn't what they say it would be.";
      $updatelog->write($message) or warn $logger::errstr;

      unlink ${vps_lock_fname};
      die $message . "\n";
    }
    # ----------------------------------------------------------------
    # check if any avast programs are using the current VPS file
    # ----------------------------------------------------------------
    my $command = 'avastvpsreload -n';
    my $avastvpscount = '';
    $avastvpscount = `$command 2>&1`;
    chomp($avastvpscount);

    $command = 'avastvpsreload';
    my @reloadarguments = (
      "-d \"$config->{-avast4tempdir}\"",
      "-l \"$logfilename\"",
    );

    # add `-a' option for root user    
    my $logname = '';
    $logname = (getpwuid($<))[0] || getlogin || "unknown";
    if ($logname eq "root") {
      unshift @reloadarguments, "-a";
    }

    # return output from 'avastvpsreload -n' should match 
    #if ($avastvpscount =~ /count of VPS allocations = (\d+)/) {
    if ($avastvpscount =~ / = (\d+)/) {
      my $avastvpscount = $1;
      unless ($avastvpscount) {
        # add -u option
        unshift @reloadarguments, "-u";
      }
    } 
    else {
      my $message = "Error: Could not determine the number of running" 
                  . " avast programs. '$command': '$avastvpscount'" ;
      $updatelog->write($message) or warn $logger::errstr;

      unlink ${vps_lock_fname};
      die $message . "\n";
    }

    $command = join " ",($command,@reloadarguments);
    print "Reloading VPS\n" if $verbose;
    print "$command\n" if $debug;
    my $reload = `$command 2>&1`;
    if ($?) {
      chomp( $reload );
      $updatelog->write("Error reloading VPS: $reload"); 
      warn "Error reloading VPS: $reload\n";
    }
    else {
      print "Reloaded VPS\n" if $verbose;
    }
  }
  else {
        $updatelog->write("WARNING: cannot download latest VPS file") 
          or warn $logger::errstr;

        unlink ${vps_lock_fname};
        die "WARNING: cannot download latest VPS file\n";
  }
}

my $res = `avastvpsreload -V 2>&1`;
$res =~ /VPS: (.*)\n/;
$updatelog->write("Current VPS version: $1") or warn $logger::errstr;
print "Current VPS version: $1\n"  if $verbose;

$updatelog->write("----- $Script ended -----") 
  or warn $logger::errstr; 

# remove lock file
unlink ${vps_lock_fname};

## -- end of program --
exit(0);

# --------------------------------------------------------------------
# package logger.
# handles the logfile
# --------------------------------------------------------------------
package logger;
use strict;
use File::Copy;
use File::Basename;

# Global error-variable
$logger::errstr = '';

sub new ($;$) {
  my ($package, $logfilename, $caller) = @_;
  unless (defined $caller) {
    $caller = basename((caller())[1]);
  }
  my %self = (
    -logfile => $logfilename,        
    -caller => $caller,
  );
  bless \%self, $package;
  return \%self;
}

sub logfile (;$) {
  my $this = shift;
  my $file = shift;

  if (not defined $file or not $file) {
    return $this->{'-logfile'};
  } 
  else {
    $this->{'-logfile'} = $file;
  }
}

# --------------------------------------------------------------------
# write to the logfile
# expects an array of lines to be printed to the logfile
# creates a new empty logfile if the logfile doesn't exist
# (using root's default umask (on my trustix machine 0077) )
sub write (;@)
{
  my $this = shift;
  my @lines = @_;

  # No logfile defined?
  if (not defined $this->{'-logfile'} or not $this->{'-logfile'}) {
    $logger::errstr = "No logfile specified";
    return;
  }

  # Open the log
  open LOG, ">>$this->{'-logfile'}"
    or $logger::errstr = "Can't open logfile $this->{'-logfile'}: $!"
    and return;

  # Timestamp, programname and pid for the loglines 
  my $timestamp = "[" . localtime() . "] " . $this->{-caller} . "[" . $$ . "]: ";

  # Print the loglines
  for (@lines) {
     # a real purist should check if the print function is
     # successfull. I think a check at CLOSE should be sufficient.
     print LOG "$timestamp $_\n";
  }

  # Close the logfile. This will fail if we're out of diskspace. 
  close LOG 
   or $logger::errstr = "Could not close the logfile:'" 
                      . $this->{'-logfile'}. "':$!"
   and return;
  return 1;
}


# --------------------------------------------------------------------
# check the size of the logfile
# if it exceeds the size in bytes given (default 1048576)
# it will be moved to $this->{'-logfile'}.old
sub maxsize (;$) 
{
  my $this = shift;
  my $maxsize = shift || 1048576;

  # No logfile defined?
  if (not defined $this->{'-logfile'} or not $this->{'-logfile'}) {
    $logger::errstr = "No logfile specified";
    return;
  }
  $this->write();
  my $size = (lstat($this->{'-logfile'}))[7];
  if ($size > $maxsize)
  {
    unless (move($this->{'-logfile'},$this->{'-logfile'} . ".old")) { 
       $logger::errstr = "Could not rotate logfile '" 
                       . $this->{'-logfile'} . "':$!";
       return undef;
    }
  }
  return 1;
}

1;
